#ifndef MOUSE_RECT_HPP
#define MOUSE_RECT_HPP

#include "opencv2/opencv.hpp"

enum TrackState { TRACK_OFF, TRACK_INIT, TRACK_ON, TRACK_SELECT };

struct MouseParam {
    cv::RotatedRect *trectp;
    enum TrackState *tstatep;
};

void onMouse(int event, int x, int y, int flags, void *param)
{
    static cv::Point topleft;

    struct MouseParam *mpp = (struct MouseParam *)param;

    if (*(mpp->tstatep) == TRACK_SELECT) {
        if (x > topleft.x && y > topleft.y) {
            mpp->trectp->center = cv::Point2f((float)(x + topleft.x) / 2,
                                              (float)(y + topleft.y) / 2);
            mpp->trectp->size = cv::Size2f((float)(x - topleft.x),
                                           (float)(y - topleft.y));
        } else {
            mpp->trectp->center = cv::Point2f((float)topleft.x,
                                              (float)topleft.y);
            mpp->trectp->size = cv::Size2f(0.0f, 0.0f);
        }
    }

    switch (event) {
    case CV_EVENT_LBUTTONDOWN:
        topleft = cv::Point(x, y);
        mpp->trectp->center = cv::Point2f((float)x, (float)y);
        mpp->trectp->size = cv::Size2f(0.0f, 0.0f);
        mpp->trectp->angle = 0.0f;
        *(mpp->tstatep) = TRACK_SELECT;
        break;
    case CV_EVENT_LBUTTONUP:
        if (mpp->trectp->size.width > 0 && mpp->trectp->size.height > 0) {
            *(mpp->tstatep) = TRACK_INIT;
        } else {
            *(mpp->tstatep) = TRACK_OFF;
        }
        break;
    }
}

#endif /* MOUSE_RECT_HPP */
